package BSS.Controllers;

import BSS.*;

import java.util.*;

public class CashMachineController extends BaseController
{
    private int _y6;
    private LinkedList _query6;
    final static Set Acceptable_Event = new TreeSet();
    final static Set Acceptable_Message = new TreeSet();

    private int _MC_id;

    static
    {
        Acceptable_Event.add(new Integer(EVENT_PUSH));
        Acceptable_Event.add(new Integer(EVENT_GLASS_BREAK));

        Acceptable_Message.add(new Integer(MSG_CASHMACHINE_SEC_ON));
        Acceptable_Message.add(new Integer(MSG_CASHMACHINE_SEC_ON));
    }

    //--- Public section ---
    public CashMachineController(int id, Logger logger, Device device)
    {
        super(id, logger, device);
        _query6 = new LinkedList();
    }

    public String toString()
    {
        return "CashMachine#" + ID();
    }

    //--- Controller interface methods ---
    public void init()
    {
        _query6.clear();
        _y6 = 0;
        Map conf = sysLoadConfig();
        _MC_id = ((Integer)conf.get("MC")).intValue();
    }

    public void run()
    {
        if (!_query6.isEmpty())
        {
            Integer event = (Integer)_query6.removeFirst();
            A6(event.intValue());
        }
        // No sensors to check
    }

    public void envOnEvent(int event_type)
    {
        if (Acceptable_Event.contains(new Integer(event_type)))
        {
            _query6.addLast(new Integer(event_type));
        }
    }

    public Message msgOnReceive(Message msg)
    {
        if (Acceptable_Message.contains(new Integer(msg.code)))
        {
            _query6.addLast(new Integer(msg.code));
        }
        return null;
    }

    //--- Automaton methods ---
    private void A6(int e)
    {
        log_start("A6", _y6, e);
        int y_now = _y6;

        switch (y_now)
        {
          case 0:
            if (e == 3 || e == 10)
            { z28();                                        _y6 = 1; }
            else
            if (e == 17)
            {                                               _y6 = 1; }
          break;

          case 1:
            if (e == 3 || e == 10)
            { z28();                                                 }
            else
            if (e == 18)
            {                                               _y6 = 0; }
          break;
        }

        if (_y6 != y_now)
        {
            switch (_y6)
            {
              case 0:
                z38();
              break;

              case 1:
                z37();
              break;
            }
        }

        log_end("A7", _y6);
    }

    private void z28()
    {
        log_z("z28", "  ");
        msgSend(_MC_id, MSG_RAID);
    }

    private void z37()
    {
        log_z("z37", " ");
        envEnergize(1);
    }

    private void z38()
    {
        log_z("z38", "  ");
        envUnenergize(1);
    }
}
