package BSS.Controllers;

import BSS.*;
import BSS.Room;

import java.util.*;

public class MainController extends BaseController
{
    private int _y0;
    private LinkedList _query0;
    final static Set Acceptable_Message = new TreeSet();
    final static Set Acceptable_SMS = new TreeSet();

    private Map _rooms;

    private Map _cashmachines;
    private Map _cashdesks;
    private Map _parkings;
    private Map _services;

    private Map _passages;
    private Map _doors;
    private Map _gateways;
    private Map _entries;

    private Message _msg;
    private SMS _sms;

    static
    {
        Acceptable_Message.add(new Integer(MSG_BURGLARY));
        Acceptable_Message.add(new Integer(MSG_INTRUSION));
        Acceptable_Message.add(new Integer(MSG_ROBBERY));
        Acceptable_Message.add(new Integer(MSG_RAID));
        Acceptable_Message.add(new Integer(MSG_UNEXPECT_LIFT_CALL));
        Acceptable_Message.add(new Integer(MSG_FIRE));
        Acceptable_Message.add(new Integer(MSG_HIJACK));

        Acceptable_SMS.add(new Integer(SMS_SMOKE_IN_ROOM));
        Acceptable_SMS.add(new Integer(SMS_LOCK_ENTRY));
        Acceptable_SMS.add(new Integer(SMS_OPEN_ENTRY));
        Acceptable_SMS.add(new Integer(SMS_SEC_ON));
        Acceptable_SMS.add(new Integer(SMS_SEC_OFF));
    }

    //--- Public section ---
    public MainController(int id, Logger logger, Device device)
    {
        super(id, logger, device);
        _query0 = new LinkedList();
    }

    public String toString()
    {
        return "MC#" + ID();
    }

    //--- Controller interface methods ---
    public void init()
    {
        _query0.clear();
        _y0 = 0;
        Map conf = sysLoadConfig();

        _rooms = (Map)conf.get("Rooms");

        _doors = (Map)conf.get("Doors");
        _gateways = (Map)conf.get("Gateways");
        _passages = (Map)conf.get("Passages");
        _entries = new TreeMap();
        _entries.putAll(_doors);
        _entries.putAll(_gateways);
        _entries.putAll(_passages);

        _cashmachines = (Map)conf.get("CashMachines");
        _cashdesks = (Map)conf.get("CashDesks");
        _parkings = (Map)conf.get("Parkings");
        _services = new TreeMap();
        _services.putAll(_cashmachines);
        _services.putAll(_cashdesks);
        _services.putAll(_parkings);
    }

    public void run()
    {
        //_msg = null;
        //_sms = null;
        if (!_query0.isEmpty())
        {
            Object o = _query0.removeFirst();
            if (o instanceof Message)
            {
                _msg = (Message)o;
                A0(_msg.code);
            }
            else if (o instanceof SMS)
            {
                _sms = (SMS)o;
                A0(_sms.code);
            }
        }
        else if (needCheck_A0())
        {
            A0(0);
        }
    }

    public Message msgOnReceive(Message msg)
    {
        if (Acceptable_Message.contains(new Integer(msg.code)))
        {
            _query0.addLast(msg);
        }
        return null;
    }

    public void smsOnReceive(SMS sms)
    {
        if (Acceptable_SMS.contains(new Integer(sms.code)))
        {
            _query0.addLast(sms);
        }
    }

    //--- Automaton methods ---
    private void A0(int e)
    {
        log_start("A0", _y0, e);
        int y_now = _y0;

        switch (y_now)
        {
          case 0:
            boolean xx27 = (e == 25) ? x27() : false;
            if (e == 25 && xx27)
            { z44();                                        _y0 = 1; }
            else
            if (e == 25 && !xx27)
            { z45();                                                 }
            else
            if (e == 26)
            {                                               _y0 = 3; }
            else
            if (e == 27)
            { z45(); z48(); z49(); z50(); z21();                     }
            else
            if (e == 28)
            {                                               _y0 = 4; }
            else
            if (e == 29)
            { z45(); z53();                                          }
            else
            if (e == 30)
            { z45(); z48(); z54();                                   }
            else
            if (e == 31)
            { z55(); z45(); z56();                                   }
            else
            if (e == 32)
            { z57();                                                 }
            else
            if (e == 33)
            { z58();                                                 }
            else
            if (e == 38)
            { z59();                                                 }
            else
            if (e == 39)
            { z60(); z62();                                          }
            else
            if (e == 40)
            { z61();                                                 }
            else
            if (e == 41)
            { z62();                                                 }
            if (e == 42)
            { z63();                                                 }
          break;

          case 1:
            boolean xx23 = x23();
            if (xx23)
            { z45(); z46();                                 _y0 = 0; }
            else
            if (!xx23)
            {                                               _y0 = 2; }
          break;

          case 2:
            boolean xx24 = x24();
            boolean xx25 = x25();
            if (!xx24 && xx25)
            { z18(); z45();                                 _y0 = 0; }
            else
            if (xx24 && xx25)
            { z20(); z45();                                 _y0 = 0; }
            else
            if (xx24 && !xx25)
            { z19(); z45();                                 _y0 = 0; }
            else
            if (!xx24 && !xx25)
            { z47();                                        _y0 = 0; }
          break;

          case 3:
            boolean xx13 = x13();
            if (xx13)
            { z21();                                        _y0 = 0; }
            else
            if (!xx13)
            {                                               _y0 = 0; }
          break;

          case 4:
            boolean xx26 = x26();
            if (xx26)
            { z55(); z52();                                 _y0 = 0; }
            else
            if (!xx26)
            { z52();                                        _y0 = 0; }
          break;
        }

        if (_y0 != y_now)
        {
            switch (_y0)
            {
              case 3:
                z45(); z48();
              break;

              case 4:
                z45(); z51();
              break;
            }
        }

        log_end("A0", _y0);
    }

    private boolean x13()
    {
        Room room = getRoom(_msg.src_id);
        boolean res = (null != room) && isRoomCritical(room.getType());
        log_x("x13", res, "   ");
        return res;
    }

    private boolean x23()
    {
        boolean res =  getEntry(_msg.src_id).isOutside();
        log_x("x23", res, " ");
        return res;
    }

    private boolean x24()
    {
        boolean res = msgRoomOnGuard(getEntryFirstRoom(_msg.src_id).ID());
        log_x("x24", res, "   ");
        return res;
    }

    private boolean x25()
    {
        boolean res = msgRoomOnGuard(getEntrySecondRoom(_msg.src_id).ID());
        log_x("x25", res, "   ");
        return res;
    }

    private boolean x26()
    {
        boolean res = _services.containsKey(new Integer(_msg.src_id));
        log_x("x26", res, "  ");
        return res;
    }

    private boolean x27()
    {
        Integer entry_id = new Integer(_msg.src_id);
        boolean res = _doors.containsKey(entry_id) || _gateways.containsKey(entry_id);
        log_x("x27", res, " /");
        return res;
    }

    private void z18()
    {
        log_z("z18", "   ");
        vbd(getEntryFirstRoom(_msg.src_id));
    }

    private void z19()
    {
        log_z("z19", "   ");
        vbd(getEntrySecondRoom(_msg.src_id));
    }

    private void z20()
    {
        log_z("z20", "    ");
        int floor1 = getEntryFirstRoom(_msg.src_id).getFloor();
        int floor2 = getEntrySecondRoom(_msg.src_id).getFloor();
        obd(floor1);
        if (floor2 != floor1)
        {
            obd(floor2);
        }
    }

    private void z21()
    {
        log_z("z21", " ");
        smsSend("SpecGuard", _msg.src_id, _msg.code);
    }

    private void z44()
    {
        log_z("z44", "    ");
        //TODO
    }

    private void z45()
    {
        log_z("z45", " ");
        smsSend("Guard", _msg.src_id, _msg.code);
    }

    private void z46()
    {
        log_z("z46", "  3 ");
        msgSend(getEntryFirstRoom(_msg.src_id).ID(), MSG_CAMERA_ON_AUTO_OFF);
    }

    private void z47()
    {
        log_z("z47", "   3 ");
        msgSend(getEntryFirstRoom(_msg.src_id).ID(), MSG_CAMERA_ON_AUTO_OFF);
        msgSend(getEntrySecondRoom(_msg.src_id).ID(), MSG_CAMERA_ON_AUTO_OFF);
    }

    private void z48()
    {
        log_z("z48", "-");
        smsSend("Owner", _msg.src_id, _msg.code);
    }

    private void z49()
    {
        log_z("z49", "   ");
        int floor = getRoom(_msg.src_id).getFloor();
        for (Iterator iterator = _rooms.values().iterator(); iterator.hasNext();)
        {
            Room room = (Room) iterator.next();
            if (floor == room.getFloor())
            {
                msgSend(room.ID(), MSG_CAMERA_ON_AUTO_OFF);
            }
        }
    }

    private void z50()
    {
        log_z("z50", "  ");
        obd(_msg.src_id);
    }

    private void z51()
    {
        log_z("z51", "");
        nbd();
    }

    private void z52()
    {
        log_z("z52", "   ,    ");
        for (Iterator iterator = _rooms.values().iterator(); iterator.hasNext();)
        {
            Room room = (Room) iterator.next();
            if (Room.CLIENTS_ROOM == room.getType() &&
                !msgRoomOnGuard(room.ID()) && !msgRoomHasPeople(room.ID()))
            {
                vbd(room);
            }
        }
    }

    private void z53()
    {
        log_z("z53", " ,     ");
        obd(_msg.data);
    }

    private void z54()
    {
        log_z("z54", "  ");
        smsSend("Firehouse", _msg.src_id, _msg.code);
    }

    private void z55()
    {
        log_z("z55", "    ");
        //TODO
    }

    private void z56()
    {
        log_z("z56", "  ");
        vbd(getService(_msg.src_id).getRoom());
    }

    private void z57()
    {
        log_z("z57", "   ");
        msgSend(_sms.object, MSG_SMOKE_ON);
    }

    private void z58()
    {
        log_z("z58", " ");
        Integer ID = new Integer(_sms.object);
        if (_doors.containsKey(ID))
        {
            msgSend(ID.intValue(), MSG_LOCK_DOOR);
        }
        else if (_gateways.containsKey(ID))
        {
            msgSend(ID.intValue(), MSG_CLOSE_GATEWAY);
        }
    }

    private void z59()
    {
        log_z("z59", " ");
        Integer ID = new Integer(_sms.object);
        if (_doors.containsKey(ID))
        {
            msgSend(ID.intValue(), MSG_UNLOCK_DOOR);
        }
        else if (_gateways.containsKey(ID))
        {
            msgSend(ID.intValue(), MSG_OPEN_GATEWAY);
        }
    }

    private void z60()
    {
        log_z("z60", "   ");
        Integer ID = new Integer(_sms.object);
        if (_rooms.containsKey(ID))
        {
            msgSend(ID.intValue(), MSG_ROOM_SEC_ON);
        }
        else if (_cashmachines.containsKey(ID))
        {
            msgSend(ID.intValue(), MSG_CASHMACHINE_SEC_ON);
        }
        else if (_cashdesks.containsKey(ID))
        {
            msgSend(ID.intValue(), MSG_CASHDESK_SEC_ON);
        }
    }

    private void z61()
    {
        log_z("z61", "   ");
        Integer ID = new Integer(_sms.object);
        if (_rooms.containsKey(ID))
        {
            msgSend(ID.intValue(), MSG_ROOM_SEC_OFF);
        }
        else if (_cashmachines.containsKey(ID))
        {
            msgSend(ID.intValue(), MSG_CASHMACHINE_SEC_OFF);
        }
        else if (_cashdesks.containsKey(ID))
        {
            msgSend(ID.intValue(), MSG_CASHDESK_SEC_OFF);
        }
        else if (_parkings.containsKey(ID))
        {
            msgSend(ID.intValue(), MSG_PARKING_SEC_OFF);
        }
    }

    private void z62()
    {
        log_z("z62", "   ");
        if (_rooms.containsKey(new Integer(_sms.object)))
        {
            msgSend(_sms.object, MSG_FIRE_SEC_ON);
        }
    }

    private void z63()
    {
        log_z("z63", "   ");
        if (_rooms.containsKey(new Integer(_sms.object)))
        {
            msgSend(_sms.object, MSG_FIRE_SEC_OFF);
        }
    }

    //--- Methods for private usage ---
    private Room getRoom(int room_id)
    {
        return (Room)_rooms.get(new Integer(room_id));
    }

    private Service getService(int service_id)
    {
        return (Service)_services.get(new Integer(service_id));
    }

    private Entry getEntry(int entry_id)
    {
        return (Entry)_entries.get(new Integer(entry_id));
    }

    private Room getEntryFirstRoom(int entry_id)
    {
        return getEntry(entry_id).getFirstRoom();
    }

    private Room getEntrySecondRoom(int entry_id)
    {
        return getEntry(entry_id).getSecondRoom();
    }

    private boolean isRoomCritical(int room_type)
    {
        return (Room.STAFF_ROOM == room_type) || (Room.STORAGE == room_type);
    }

    private void vbd(Room room)
    {
        for (Iterator it = room.getEntries().iterator(); it.hasNext();)
        {
            Entry entry = (Entry)it.next();
            int entry_type = entry.getType();
            if (Entry.DOOR == entry_type)
            {
                msgSend(entry.ID(), MSG_LOCK_DOOR);
            }
            else if (Entry.GATEWAY == entry_type)
            {
                msgSend(entry.ID(), MSG_CLOSE_GATEWAY);
            }
        }
    }

    private void obd(int floor)
    {
        for (Iterator iterator = _doors.values().iterator(); iterator.hasNext();)
        {
            Entry door = (Entry) iterator.next();
            if (floor == getEntryFirstRoom(door.ID()).getFloor() ||
                floor == getEntrySecondRoom(door.ID()).getFloor())
            {
                msgSend(door.ID(), MSG_LOCK_DOOR);
            }
        }
        for (Iterator iterator = _gateways.values().iterator(); iterator.hasNext();)
        {
            Entry gateway = (Entry) iterator.next();
            if (floor == getEntryFirstRoom(gateway.ID()).getFloor() ||
                floor == getEntrySecondRoom(gateway.ID()).getFloor())
            {
                msgSend(gateway.ID(), MSG_CLOSE_GATEWAY);
            }
        }
    }

    private void nbd()
    {
        for (Iterator iterator = _doors.values().iterator(); iterator.hasNext();)
        {
            Entry door = (Entry) iterator.next();
            if (door.isOutside())
            {
                msgSend(door.ID(), MSG_LOCK_DOOR);
            }
        }
        for (Iterator iterator = _gateways.values().iterator(); iterator.hasNext();)
        {
            Entry gateway = (Entry) iterator.next();
            if (gateway.isOutside())
            {
                msgSend(gateway.ID(), MSG_CLOSE_GATEWAY);
            }
        }
    }

    private boolean needCheck_A0()
    {
        return (1 <= _y0) && (4 >= _y0);
    }
}
