package BSS.Controllers;

import BSS.*;

import java.util.*;

public class ParkingController extends BaseController
{
    private int _y8;
    private int _y8_1;
    private LinkedList _query8;
    final static Set Acceptable_Interrupt = new TreeSet();
    final static Set Acceptable_Message = new TreeSet();

    private int _MC_id;

    private Random _rand;
    static int MAX_PIN_FAIL = 5;
    private String _PIN;
    private String _input_PIN;

    static
    {
        Acceptable_Interrupt.add(new Integer(EVENT_PUSH));
        Acceptable_Interrupt.add(new Integer(EVENT_ALARM_ON_BTN));
        Acceptable_Interrupt.add(new Integer(EVENT_ALARM_OFF_BTN));

        Acceptable_Message.add(new Integer(MSG_PARKING_SEC_OFF));
    }

    //--- Public section ---
    public ParkingController(int id, Logger logger, Device device)
    {
        super(id, logger, device);
        _query8 = new LinkedList();
        _rand = new Random();
    }

    public String toString()
    {
        return "Parking#" + ID();
    }

    //--- Controller interface methods ---
    public void init()
    {
        _query8.clear();
        _y8 = 0;
        _y8_1 = 0;
        Map conf = sysLoadConfig();
        _MC_id = ((Integer)conf.get("MC")).intValue();
        _PIN = "";
    }

    public void run()
    {
        if (!_query8.isEmpty())
        {
            Integer event = (Integer)_query8.removeFirst();
            A8(event.intValue());
        }
        else if (needCheck_A8())
        {
            A8(0);
        }
    }

    public void envOnEvent(int event_type)
    {
        if (Acceptable_Interrupt.contains(new Integer(event_type)))
        {
            _query8.addLast(new Integer(event_type));
        }
    }

    public Message msgOnReceive(Message msg)
    {
        if (Acceptable_Message.contains(new Integer(msg.code)))
        {
            _query8.addLast(new Integer(msg.code));
        }
        return null;
    }

    //--- Automaton methods ---
    private void A8(int e)
    {
        log_start("A8", _y8, e);
        int y_now = _y8;

        switch (y_now)
        {
          case 0:
            if (e == 21 && x20())
            { z39(); z40(); z42();                          _y8 = 1; }
          break;

          case 1:
            boolean xx21 = x21();
            if (e == 3 || e == 10)
            { z28();                                                 }
            else
            if (e == 22 ||(e == 24 && xx21) )
            {                                               _y8 = 0; }
            else
            if (!x20())
            {                                               _y8 = 3; }
            else
            if (e == 24 && !xx21)
            {                                               _y8 = 2; }
          break;

          case 2:
            boolean xx22 = x22();
            if (xx22)
            {                                               _y8 = 3; }
            else
            if (!xx22)
            {                                               _y8 = 1; }
          break;

          case 3:
            if (e == 22)
            {                                               _y8 = 0; }
          break;
        }

        if (_y8 != y_now)
        {
            switch (_y8)
            {
              case 2:
                z43();
              break;

              case 3:
                z41();
              break;
            }
        }

        log_end("A8", _y8);
    }

    private boolean x20()
    {
        boolean res = envHasMetal(1);
        log_x("x20", res, "  ");
        return res;
    }

    private boolean x21()
    {
        _input_PIN = envReadKeyboard(1);
        boolean res =  _PIN.equals(_input_PIN);
        log_x("x21", res, "  PIN-");
        return res;
    }

    private boolean x22()
    {
        boolean res = MAX_PIN_FAIL < _y8_1;
        log_x("x22", res, " PIN-");
        return res;
    }

    private void z28()
    {
        log_z("z28", "  ");
        msgSend(_MC_id, MSG_RAID);
    }

    private void z39()
    {
        log_z("z39", "  PIN-");
        int number = 1000 + _rand.nextInt(9000);
        _PIN = String.valueOf(number);
    }

    private void z40()
    {
        log_z("z40", "  PIN-");
        envPrint(1, _PIN);
    }

    private void z41()
    {
        log_z("z41", "  ");
        msgSend(_MC_id, MSG_HIJACK);
    }

    private void z42()
    {
        log_z("z42", " ");
        _y8_1 = 0;
    }

    private void z43()
    {
        log_z("z43", " ");
        _y8_1++;
    }

    //--- Methods for private usage ---
    private boolean needCheck_A8()
    {
        return (1 == _y8) || (2 == _y8);
    }
}
