package BSS.Controllers;

import BSS.*;

import java.util.*;

public class RoomController extends BaseController
{
    private int _y4;
    private LinkedList _query4;
    final static Set A4_Acceptable_Event = new TreeSet();
    final static Set A4_Acceptable_Message = new TreeSet();

    private int _y5;
    private int _y5_1;
    private LinkedList _query5;
    final static Set A5_Acceptable_Message = new TreeSet();

    private int _MC_id;
    private int _type;
    private int _lower_floor;
    private Set _doorgates;
    private boolean _isActive;
    static int PASSIVE_LEVEL = 100;
    static int ACTIVE_LEVEL  = 200;

    static
    {
        A4_Acceptable_Event.add(new Integer(EVENT_MOTION));
        A4_Acceptable_Event.add(new Integer(EVENT_GLASS_BREAK));
        A4_Acceptable_Event.add(new Integer(EVENT_LIFT_CALL));
        A4_Acceptable_Event.add(new Integer(EVENT_ROBBERY_BTN));

        A4_Acceptable_Message.add(new Integer(MSG_ROOM_SEC_ON));
        A4_Acceptable_Message.add(new Integer(MSG_ROOM_SEC_OFF));
        A4_Acceptable_Message.add(new Integer(MSG_SMOKE_ON));
        A4_Acceptable_Message.add(new Integer(MSG_CAMERA_ON_AUTO_OFF));

        A5_Acceptable_Message.add(new Integer(MSG_FIRE_SEC_ON));
        A5_Acceptable_Message.add(new Integer(MSG_FIRE_SEC_OFF));
    }

    //--- Public section ---
    public RoomController(int id, Logger logger, Device device)
    {
        super(id, logger, device);
        _query4 = new LinkedList();
        _query5 = new LinkedList();
    }

    public String toString()
    {
        return "Room#" + ID();
    }

    //--- Controller interface methods ---
    public void init()
    {
        _query4.clear();
        _query5.clear();
        _y4 = 0;
        _y5 = 0;
        _y5_1 = 0;
        Map conf = sysLoadConfig();
        _MC_id = ((Integer)conf.get("MC")).intValue();
        _isActive = ((Boolean)conf.get("ActiveFS")).booleanValue();
        _type = ((Integer)conf.get("Type")).intValue();
        _lower_floor = (Room.LIFT == _type) ? ((Integer)conf.get("LowerFloor")).intValue() : 0;

        _doorgates = new HashSet();
        Set doors = (Set)conf.get("Doors");
        Set gateways = (Set)conf.get("Gateways");
        _doorgates.addAll(doors);
        _doorgates.addAll(gateways);
    }

    public void run()
    {
        if (!_query5.isEmpty())
        {
            Integer event = (Integer)_query5.removeFirst();
            A5(event.intValue());
        }
        else if (needCheck_A5())
        {
            A5(0);
        }
        if (!_query4.isEmpty())
        {
            Integer event = (Integer)_query4.removeFirst();
            A4(event.intValue());
        }
        else if (needCheck_A4())
        {
            A4(0);
        }
    }

    public void envOnEvent(int event_type)
    {
        if (A4_Acceptable_Event.contains(new Integer(event_type)))
        {
            _query4.addLast(new Integer(event_type));
        }
    }

    public Message msgOnReceive(Message msg)
    {
        Message response = null;

        if (REQ_ROOM_ON_GUARD == msg.code)
        {
            response = msgMakeReply(msg, isGuarded()? 1 : 0);
        }
        else if (REQ_ROOM_HAS_PEOPLE == msg.code)
        {
            response =  msgMakeReply(msg, isCrowded()? 1 : 0);
        }
        else if (A4_Acceptable_Message.contains(new Integer(msg.code)))
        {
            _query4.addLast(new Integer(msg.code));
        }
        else if (A5_Acceptable_Message.contains(new Integer(msg.code)))
        {
            _query5.addLast(new Integer(msg.code));
        }

        return response;
    }

    //--- Authomaton methods ---
    private void A4(int e)
    {
        log_start("A4", _y4, e);
        int y_now = _y4;

        switch (y_now)
        {
          case 0:
            if (e == 6 && x12())
            {                                               _y4 = 1; }
            else
            if (e == 13)
            { z46();                                                 }
            else
            if (e == 14)
            {                                               _y4 = 2; }
          break;

          case 1:
            if (e == 7)
            { z25();                                        _y4 = 0; }
            else
            if (e == 8)
            { z17();                                                 }
            else
            if (e == 9)
            { z26(); z22();                                          }
            else
            if (e == 10)
            { z6(); z22();                                           }
            else
            if (e == 12)
            { z29(); z24();                                          }
            else
            if (e == 13)
            { z46();                                                 }
            else
            if (!x12())
            { z27(); z22();                                          }
          break;

          case 2:
            if (e == 7)
            {                                               _y4 = 0; }
          break;
        }

        if (_y4 != y_now)
        {
            switch (_y4)
            {
              case 0:
                z23();
              break;

              case 2:
                z28(); z22();
              break;
            }
        }

        log_end("A4", _y4);
    }

    private void A5(int e)
    {
        log_start("A5", _y5, e);
        int y_now = _y5;

        switch (y_now)
        {
          case 0:
            if (e == 15)
            { z30();                                        _y5 = 1; }
          break;

          case 1:
            boolean xx15 = x15();
            if (e == 16)
            {                                               _y5 = 0; }
            else
            if (xx15)
            { z32();                                        _y5 = 2; }
            else
            if (!xx15 && x16())
            { z31();                                        _y5 = 2; }
          break;

          case 2:
            boolean xx17 = x17();
            boolean xx18 = x18();
            boolean xx19 = x19();
            if (xx17)
            { z34();                                        _y5 = 1; }
            else
            if (!xx17 && xx18)
            { z33(); z9();                                  _y5 = 1; }
            else
            if (!xx18 && !xx19)
            { z31(); z9();                                  _y5 = 1; }
            else
            if (!xx18 && xx19)
            {                                               _y5 = 3; }
          break;

          case 3:
            if (e == 16)
            {                                               _y5 = 0; }
          break;
        }

        if (_y5 != y_now)
        {
            switch (_y5)
            {
              case 0:
                z36();
              break;

              case 3:
                z35();
              break;
            }
        }

        log_end("A5", _y5);
    }

    private boolean x12()
    {
        boolean res = true;

        for (Iterator iterator = _doorgates.iterator(); iterator.hasNext();)
        {
            Integer entryID = (Integer) iterator.next();
            if (!msgEntryIsLocked(entryID.intValue()))
            {
                res = false;
                break;
            }
        }

        log_x("x12", res, "/     ");
        return res;
    }

    private boolean x15()
    {
        boolean res = envHotAndSmoky(1);
        log_x("x15", res, "       ");
        return res;
    }

    private boolean x16()
    {
        boolean res = _y5_1 > 0;
        log_x("x16", res, "  ");
        return res;
    }

    private boolean x17()
    {
        boolean res = _y5_1 < PASSIVE_LEVEL;
        log_x("x17", res, "    ");
        return res;
    }

    private boolean x18()
    {
        boolean res = _y5_1 < ACTIVE_LEVEL;
        log_x("x18", res, "    ");
        return res;
    }

    private boolean x19()
    {
        boolean res = _isActive;
        log_x("x19", res, "  ");
        return res;
    }

    private void z6()
    {
        log_z("z6", "  ");
        msgSend(_MC_id, MSG_BURGLARY);
    }

    private void z9()
    {
        log_z("z9", "  ");
        msgSend(_MC_id, MSG_FIRE);
    }

    private void z17()
    {
        log_z("z17", " ");
        envSmokeOn(1);
    }

    private void z22()
    {
        log_z("z22", " ,  ..");
        envCameraOn(1);
        envInterferenceOn(1);
    }

    private void z23()
    {
        log_z("z23", " ,  ..");
        envCameraOff(1);
        envInterferenceOff(1);
    }

    private void z24()
    {
        log_z("z24", "  ");
        envStopLift(1);
    }

    private void z25()
    {
        log_z("z25", " ");
        envSmokeOff(1);
    }

    private void z26()
    {
        log_z("z26", "  ");
        msgSend(_MC_id, MSG_INTRUSION);
    }

    private void z27()
    {
        log_z("z27", "  ");
        msgSend(_MC_id, MSG_ROBBERY);
    }

    private void z28()
    {
        log_z("z28", "  ");
        msgSend(_MC_id, MSG_RAID);
    }

    private void z29()
    {
        log_z("z29", "    ");
        int floor = _lower_floor + envReadLiftCaller(1);
        msgSend(_MC_id, MSG_UNEXPECT_LIFT_CALL, floor);
    }

    private void z30()
    {
        log_z("z30", " ");
        _y5_1 = 0;
    }

    private void z31()
    {
        log_z("z31", " ");
        _y5_1--;
    }

    private void z32()
    {
        log_z("z32", " ");
        _y5_1++;
    }

    private void z33()
    {
        log_z("z33", "     ");
        envOn_LED(1);
        msgSend(_MC_id, MSG_FIRE);
    }

    private void z34()
    {
        log_z("z34", "     ");
        envOff_LED(1);
    }

    private void z35()
    {
        log_z("z35", "  ");
        envFireExtinguisherOn(1);
    }

    private void z36()
    {
        log_z("z36", "  ");
        envFireExtinguisherOff(1);
    }

    private void z46()
    {
        log_z("z46", "  3 ");
        envCameraOnAndAutoOff(1);
    }

    //--- Methods for private usage ---
    private boolean isGuarded()
    {
        return 1 == _y4;
    }

    private boolean isCrowded()
    {
        return envHasBadVolume(1);
    }

    private boolean needCheck_A4()
    {
        return (1 == _y4);
    }

    private boolean needCheck_A5()
    {
        return (1 == _y5) || (2 == _y5);
    }
}
