package BSS.Devices;

import BSS.*;

import java.util.Map;

public class DoorDevice extends BaseDevice
{
    byte _contact_sensor_status;
    byte _smartcard_status;
    String _smartcard_data;

    public DoorDevice(int id, Logger logger, Map configuration, Communicator communicator)
    {
        super(id, logger, configuration, communicator);
        _contact_sensor_status = 0;
        _smartcard_status = 0;
        _smartcard_data = "";
    }

    public String toString()
    {
        return "Door#" + ID();
    }

    //--- Device interface methods ---
    public byte getStatus(int device_type, int device_id)
    {
        boolean read = false;
        byte value = 0;
        switch (device_type)
        {
          case DEV_CONTACT_SENSOR:
            if (1 == device_id)
            {
                value = _contact_sensor_status;
                read = true;
            }
          break;

          case DEV_SMARTCARD_DRIVE:
            if (1 == device_id)
            {
                value = _smartcard_status;
                read = true;
            }
          break;
        }
        log_get_status(read, device_type, device_id, value);
        return value;
    }

    public byte[] readData(int device_type, int device_id)
    {
        boolean read = false;
        byte[] value = new byte[0];
        switch (device_type)
        {
          case DEV_SMARTCARD_DRIVE:
            if (1 == device_id)
            {
                value = _smartcard_data.getBytes();
                read = true;
            }
          break;
        }
        log_read_data(read, device_type, device_id, value);
        return value;
    }

    public void setStatus(int device_type, int device_id, byte value)
    {
        boolean write = false;
        switch (device_type)
        {
          case DEV_LED:
          case DEV_LOCK:
          case DEV_SMARTCARD_DRIVE:
            if (1 == device_id)
            {
                write = true;
            }
          break;
        }
        log_set_status(write, device_type, device_id, value);
    }

    public void sendSMS(SMS sms)
    {
        log(1, _con.toString() + " attempts to send " + sms);
    }

    //--- Host interafce methods --
    public void updateStatus(int device_type, int device_id, byte value)
    {
        boolean updated = false;
        switch (device_type)
        {
          case DEV_CONTACT_SENSOR:
            if (1 == device_id)
            {
                _contact_sensor_status = value;
                updated = true;
            }
          break;

          case DEV_SMARTCARD_DRIVE:
            if (1 == device_id)
            {
                _smartcard_status = value;
                updated = true;
            }
          break;
        }
        log_update_status(updated, device_type, device_id, value);
    }

    public void updateData(int device_type, int device_id, byte[] value)
    {
        boolean updated = false;
        switch (device_type)
        {
          case DEV_SMARTCARD_DRIVE:
            if (1 == device_id)
            {
                _smartcard_data = new String(value);
                updated = true;
            }
          break;
        }
        log_update_data(updated, device_type, device_id, value);
    }

    public void receiveSMS(SMS sms)
    {
        log(1, "Unexpected SMS: " + sms);
    }
}
