package BSS.Devices;

import BSS.*;

import java.util.Map;

public class GatewayDevice extends BaseDevice
{
    boolean _hasPhoto;
    byte _photo_sensor_status;
    byte[] _contact_sensor_status;

    public GatewayDevice(int id, Logger logger, Map configuration, Communicator communicator, boolean hasPhotocell)
    {
        super(id, logger, configuration, communicator);
        _hasPhoto = hasPhotocell;
        _contact_sensor_status = new byte[2];
        if (hasPhotocell)
        {
            _photo_sensor_status = 0;
        }
    }

    public String toString()
    {
        return "Gateway#" + ID();
    }

    //--- Device interface methods ---
    public byte getStatus(int device_type, int device_id)
    {
        boolean read = false;
        byte value = 0;
        switch (device_type)
        {
          case DEV_CONTACT_SENSOR:
            if (1 <= device_id && device_id <= 2)
            {
                value = _contact_sensor_status[device_id-1];
                read = true;
            }
          break;

          case DEV_PHOTO_SENSOR:
            if (_hasPhoto && 1 == device_id)
            {
                value = _photo_sensor_status;
                read = true;
            }
          break;
        }
        log_get_status(read, device_type, device_id, value);
        return value;
    }

    public void setStatus(int device_type, int device_id, byte value)
    {
        boolean write = false;
        switch (device_type)
        {
          case DEV_LOCK:
          case DEV_MOTOR:
            if (1 == device_id)
            {
                write = true;
            }
          break;
        }
        log_set_status(write, device_type, device_id, value);
    }

    public void sendSMS(SMS sms)
    {
        log(1, _con.toString() + " attempts to send " + sms);
    }

    //--- Host interface methods ---
    public void updateStatus(int device_type, int device_id, byte value)
    {
        boolean updated = false;
        switch (device_type)
        {
          case DEV_CONTACT_SENSOR:
            if (1 >= device_id && device_id <= 2)
            {
                _contact_sensor_status[device_id-1] = value;
                updated = true;
            }
          break;

          case DEV_PHOTO_SENSOR:
            if (_hasPhoto && 1 == device_id)
            {
                _photo_sensor_status = value;
                updated = true;
            }
          break;
        }
        log_update_status(updated, device_type, device_id, value);
    }

    public void receiveSMS(SMS sms)
    {
        log(1, "Unexpected SMS: " + sms);
    }
}
