package BSS.Simulator;

import BSS.*;

import java.util.Map;
import java.util.TreeMap;

public class MessageCenter implements Communicator, EnvironmentCodes
{
    public final static Map MESSAGE_CODE_NAMES = new TreeMap();
    static
    {
        MESSAGE_CODE_NAMES.put(new Integer(MSG_LOCK_DOOR), "LockDoor");
        MESSAGE_CODE_NAMES.put(new Integer(MSG_UNLOCK_DOOR), "UnlockDoor");
        MESSAGE_CODE_NAMES.put(new Integer(MSG_CLOSE_GATEWAY), "CloseGateway");
        MESSAGE_CODE_NAMES.put(new Integer(MSG_OPEN_GATEWAY), "OpenGateway");
        MESSAGE_CODE_NAMES.put(new Integer(MSG_ROOM_SEC_ON), "TurnOnRoomSec");
        MESSAGE_CODE_NAMES.put(new Integer(MSG_ROOM_SEC_OFF), "TurnOffRoomSec");
        MESSAGE_CODE_NAMES.put(new Integer(MSG_SMOKE_ON), "TurnOnSmoke");
        MESSAGE_CODE_NAMES.put(new Integer(MSG_CAMERA_ON_AUTO_OFF), "TurnOnAutoOffCam");
        MESSAGE_CODE_NAMES.put(new Integer(MSG_FIRE_SEC_ON), "TurnOnFireSec");
        MESSAGE_CODE_NAMES.put(new Integer(MSG_FIRE_SEC_OFF), "TurnOnFireSecOff");
        MESSAGE_CODE_NAMES.put(new Integer(MSG_CASHMACHINE_SEC_ON), "TurnOnCashMachineSec");
        MESSAGE_CODE_NAMES.put(new Integer(MSG_CASHMACHINE_SEC_OFF), "TurnOffCashMachineSec");
        MESSAGE_CODE_NAMES.put(new Integer(MSG_CASHDESK_SEC_ON), "TurnOnCashDeskSec");
        MESSAGE_CODE_NAMES.put(new Integer(MSG_CASHDESK_SEC_OFF), "TurnOffCashDeskSec");
        MESSAGE_CODE_NAMES.put(new Integer(MSG_PARKING_SEC_OFF), "TurnOffParkingSec");
        MESSAGE_CODE_NAMES.put(new Integer(MSG_BURGLARY), "Burglary");
        MESSAGE_CODE_NAMES.put(new Integer(MSG_INTRUSION), "Intrusion");
        MESSAGE_CODE_NAMES.put(new Integer(MSG_ROBBERY), "Robbery");
        MESSAGE_CODE_NAMES.put(new Integer(MSG_RAID), "Raid");
        MESSAGE_CODE_NAMES.put(new Integer(MSG_UNEXPECT_LIFT_CALL), "UnexpectedLiftCall");
        MESSAGE_CODE_NAMES.put(new Integer(MSG_FIRE), "Fire");
        MESSAGE_CODE_NAMES.put(new Integer(MSG_HIJACK), "Hijack");
        MESSAGE_CODE_NAMES.put(new Integer(REQ_ROOM_ON_GUARD), "RoomOnGuard?");
        MESSAGE_CODE_NAMES.put(new Integer(REQ_ROOM_HAS_PEOPLE), "RoomHasPeople?");
        MESSAGE_CODE_NAMES.put(new Integer(REQ_ENTRY_IS_LOCKED), "EntryLocked?");
        MESSAGE_CODE_NAMES.put(new Integer(REQ_ENTRY_IS_CLOSED), "EntryClosed?");

        MESSAGE_CODE_NAMES.put(new Integer(SMS_SMOKE_IN_ROOM), "TurnOnGas");
        MESSAGE_CODE_NAMES.put(new Integer(SMS_LOCK_ENTRY), "LockEntry");
        MESSAGE_CODE_NAMES.put(new Integer(SMS_OPEN_ENTRY), "OpenEntry");
        MESSAGE_CODE_NAMES.put(new Integer(SMS_SEC_ON), "TurnOnSec");
        MESSAGE_CODE_NAMES.put(new Integer(SMS_SEC_OFF), "TurnOffSec");
        MESSAGE_CODE_NAMES.put(new Integer(SMS_FIRE_SEC_ON), "TurnOnFireSec");
        MESSAGE_CODE_NAMES.put(new Integer(SMS_FIRE_SEC_OFF), "TurnOffFireSec");
    }

    Logger _logger;
    Map    _hosts; // Integer -> Host
    Integer _MC_ID;

    public MessageCenter(Logger logger, int MC_id)
    {
        _logger = logger;
        _MC_ID = new Integer(MC_id);
    }

    public void setHosts(Map hosts)
    {
        if (null == hosts)
        {
            throw new IllegalArgumentException("Null hosts.");
        }
        _hosts = hosts;
    }

    //--- Communicator interface methods ---
    public Message sendMessage(Message msg)
    {
        Host src_host = (Host)_hosts.get(new Integer(msg.src_id));
        if (null == src_host)
        {
            log(1, "Invalid source host: " + msg);
            return null;
        }

        Host dst_host = (Host)_hosts.get(new Integer(msg.dst_id));
        if (null == dst_host)
        {
            log(1, "Invalid destination host: " + msg);
            return null;
        }

        log(3, "Message from " + src_host.toString() + " to " + src_host.toString() + " with code=" + msgCodeToString(msg.code) + " and data=" + msg.data);
        Message reply = dst_host.receiveMessage(msg);
        if (null != reply)
        {
            log(3, "Reply: " + reply);
        }
        return reply;
    }

    public void sendSMS(SMS sms)
    {
        log(3, "SMS from " + sms.from + " to " + sms.to + " with object=" + sms.object + " and code=" + msgCodeToString(sms.code));
        if ("BSS".equals(sms.to))
        {
            Host host = (Host)_hosts.get(_MC_ID);
            host.receiveSMS(sms);
        }
    }

    //--- Methods for private usage ---
    private void log(int level, String message)
    {
        _logger.log(level, message);
    }

    private String msgCodeToString(int msg_code)
    {
        String desc = (String)MESSAGE_CODE_NAMES.get(new Integer(msg_code));
        return (null == desc) ? String.valueOf(msg_code) : desc;
    }
}
